#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import argparse
import requests
from bs4 import BeautifulSoup
from datetime import datetime, timezone
from lxml import etree
import re
import urllib.parse
from xml.etree import ElementTree as ET

def parse_sitemap(path):
    tree = ET.parse(path)
    root = tree.getroot()
    ns = {'ns': 'http://www.sitemaps.org/schemas/sitemap/0.9'}
    return [el.text for el in root.findall('.//ns:loc', ns) if '/entry/' in el.text]

def clean_entry_content(html):
    soup = BeautifulSoup(html, 'html.parser')
    # (광고/스크립트/iframe/Button 제거 로직)
    for toc in soup.select('div.book-toc'): toc.decompose()
    for tag in soup.find_all(['script', 'ins', 'iframe']): tag.decompose()
    pattern = re.compile(r'ca-pub-\d+')
    for tag in soup.find_all(True):
        try:
            if tag.string and pattern.search(tag.string):
                tag.decompose(); continue
            attrs = tag.attrs or {}
            if any(pattern.search(str(v)) for v in attrs.values()):
                tag.decompose(); continue
            style = attrs.get('style','')
            if any(x in style for x in ['adsbygoogle','overflow:hidden']):
                tag.decompose(); continue
            if attrs.get('data-tistory-react-app') in ['NaverAd','Reaction']:
                tag.decompose(); continue
            if attrs.get('id')=='adsense_script':
                tag.decompose(); continue
        except:
            pass
    for btn in soup.find_all('button'): btn.decompose()
    return soup.decode_contents()

def fetch_post(url):
    headers = {'User-Agent':'Mozilla/5.0'}
    resp = requests.get(url, headers=headers, timeout=10); resp.raise_for_status()
    soup = BeautifulSoup(resp.text,'html.parser')

    # title
    title = soup.find('title').get_text().strip() if soup.find('title') else url

    # date
    date_str = ''
    if span := soup.find('span', class_='date'):
        date_str = span.get_text().strip()
    elif time_tag := soup.find('time'):
        date_str = time_tag.get('datetime','')
    try:
        m = re.match(r"(\d{4})\.\s*(\d{1,2})\.\s*(\d{1,2})\.", date_str)
        pub = datetime(*map(int, m.groups())) if m else datetime.fromisoformat(date_str)
    except:
        pub = datetime.now()

    # RSS pubDate (UTC)
    rss_pub = pub.astimezone(timezone.utc) if pub.tzinfo else pub
    pubDate = rss_pub.strftime('%a, %d %b %Y %H:%M:%S +0000')

    # content
    cont = (soup.find('div', class_='contents_style')
            or soup.find('article') or soup.find('section'))
    raw_html = cont.decode_contents() if cont else ''
    cleaned = clean_entry_content(raw_html)

    # guid
    guid_url = (soup.find("meta", property="dg:plink") or {}).get("content", url)

    return title, pubDate, pub, cleaned, url, guid_url

def build_wxr(posts, out_path, base_url, start_id=0):
    NSMAP = {
        'excerpt':"http://wordpress.org/export/1.2/excerpt/",
        'content':"http://purl.org/rss/1.0/modules/content/",
        'dc':"http://purl.org/dc/elements/1.1/",
        'wp':"http://wordpress.org/export/1.2/"
    }
    wp_ns = NSMAP['wp']

    root = etree.Element('rss', nsmap=NSMAP)
    root.set('version', '2.0')
    channel = etree.SubElement(root, 'channel')

    def add(parent, tag, text):
        el = etree.SubElement(parent, tag)
        if tag == 'title':
            el.text = etree.CDATA(text)
        elif tag == '{http://purl.org/dc/elements/1.1/}creator':
            el.text = etree.CDATA(text)
        elif tag.startswith(f'{{{wp_ns}}}'):
            el.text = text.isdigit() and text or etree.CDATA(text)
        else:
            el.text = text
        return el

    # 채널 헤더
    add(channel, 'title', 'Tistory 백업')
    add(channel, 'link', base_url)
    add(channel, 'description', '')
    add(channel, 'language', 'ko-KR')
    add(channel, '{http://wordpress.org/export/1.2/}wxr_version', '1.2')
    add(channel, '{http://wordpress.org/export/1.2/}base_site_url', base_url)
    add(channel, '{http://wordpress.org/export/1.2/}base_blog_url', base_url)

    # author
    auth = etree.SubElement(channel, '{http://wordpress.org/export/1.2/}author')
    add(auth, '{http://wordpress.org/export/1.2/}author_id', '1')
    add(auth, '{http://wordpress.org/export/1.2/}author_login', 'admin')
    add(auth, '{http://wordpress.org/export/1.2/}author_email', 'admin@blog.com')
    add(auth, '{http://wordpress.org/export/1.2/}author_display_name', 'admin')

    # posts
    for idx, (title, pubDate, pub, content, link, guid_url) in enumerate(posts, start=start_id+1):
        item = etree.SubElement(channel, 'item')
        add(item, 'title', title)
        add(item, 'link', link)
        add(item, 'pubDate', pubDate)
        add(item, '{http://purl.org/dc/elements/1.1/}creator', 'admin')

        guid_el = etree.SubElement(item, 'guid', isPermaLink="false")
        guid_el.text = guid_url

        etree.SubElement(item, 'description')

        ce = etree.SubElement(item, '{http://purl.org/rss/1.0/modules/content/}encoded')
        minified = re.sub(r'>\s+<','><', content.replace('\n','').replace('\r','').replace('\t',''))
        ce.text = etree.CDATA(minified)

        dt_wp = pub.strftime('%Y-%m-%d %H:%M:%S')
        dt_gmt = pub.astimezone(timezone.utc).strftime('%Y-%m-%d %H:%M:%S') if pub.tzinfo else dt_wp
        add(item, '{http://wordpress.org/export/1.2/}post_id', str(idx))
        add(item, '{http://wordpress.org/export/1.2/}post_date', dt_wp)
        add(item, '{http://wordpress.org/export/1.2/}post_date_gmt', dt_gmt)

        # 기타 wp 메타
        add(item, '{http://wordpress.org/export/1.2/}comment_status', 'open')
        add(item, '{http://wordpress.org/export/1.2/}ping_status', 'open')
        slug = link.rstrip('/').split('/')[-1]
        add(item, '{http://wordpress.org/export/1.2/}post_name', slug)
        add(item, '{http://wordpress.org/export/1.2/}status', 'publish')
        add(item, '{http://wordpress.org/export/1.2/}post_parent', '0')
        add(item, '{http://wordpress.org/export/1.2/}menu_order', '0')
        add(item, '{http://wordpress.org/export/1.2/}post_type', 'post')
        add(item, '{http://wordpress.org/export/1.2/}post_password', '')
        add(item, '{http://wordpress.org/export/1.2/}is_sticky', '0')

        cat = etree.Element('category', domain="category", nicename="uncategorized")
        cat.text = "Uncategorized"
        item.append(cat)

        meta = etree.SubElement(item, '{http://wordpress.org/export/1.2/}postmeta')
        add(meta, '{http://wordpress.org/export/1.2/}meta_key', '_edit_last')
        add(meta, '{http://wordpress.org/export/1.2/}meta_value', '1')

    tree = etree.ElementTree(root)
    tree.write(out_path, encoding='utf-8', pretty_print=True, xml_declaration=True)
    remove_xml_comments(out_path)
    print(f"WXR 생성 완료: {out_path}")

def remove_xml_comments(path):
    import re
    txt = open(path, 'r', encoding='utf-8').read()
    for pat in [
        r'<!--\s*System\s*-\s*START\s*-->',
        r'<!--\s*System\s*-\s*END\s*-->',
        r'<!--\s*inventory\s*-->',
        r'<!--\s*PostListinCategory\s*-\s*START\s*-->',
        r'<!--\s*PostListinCategory\s*-\s*END\s*-->'
    ]:
        txt = re.sub(pat, '', txt, flags=re.IGNORECASE)
    with open(path, 'w', encoding='utf-8') as f:
        f.write(txt)

def main():
    parser = argparse.ArgumentParser(description='Tistory → WordPress 변환기')
    parser.add_argument('--sitemap', default='sitemap.xml', help='Sitemap XML 경로')
    parser.add_argument('--output', default='wordpress_export_final.xml', help='출력 WXR 파일명')
    args = parser.parse_args()

    urls = parse_sitemap(args.sitemap)
    base = urllib.parse.urlparse(urls[0])
    base_url = f"{base.scheme}://{base.netloc}"

    posts = []
    for url in urls:
        print(f"크롤링: {url}")
        try:
            posts.append(fetch_post(url))
        except Exception as e:
            print(f"오류 발생: {e}")

    # 100개씩 분할 저장
    prefix, ext = args.output.rsplit('.', 1)
    for i in range(0, len(posts), 100):
        chunk = posts[i:i+100]
        part = i // 100 + 1
        out_file = f"{prefix}-{part:03d}.{ext}"
        build_wxr(chunk, out_file, base_url, start_id=i)

if __name__ == '__main__':
    main()
